package jemu.core.device.tape;

/**
 * Generic wave handling routines
 * Converted from "TZX to WAV Converter v0.2 (C) 2006 Francisco Javier Crespo <tzx2wav@ya.com>"
 * 
 * @author John Girvin
 *
 */
public abstract class CDT2WAVOut {
	//
	// CONSTANTS
	//
	private static  double Z80_FREQ = 3500000.0;	// Z80 frequency in Hz

	
	private static final int LOAMP    = 0x26;      		// Low Level Amplitude  (-3 dB)
	private static final int HIAMP    = 0xDA;      		// High Level Amplitude (-3 dB)
    protected boolean med = false;

	//
	// INSTANCE VARIABLES
	//
	private byte[] buf       = null;		// output buffer
	private double frequency = 44100.0;	// wave data frequency
	private double cycle     = 0.0;		// Z80 cycles per wave sample
	
	private int amp = LOAMP;				// current amplitude

	
	//
	// ACCESSOR METHODS
	//
	protected byte[] getBuf() {
		return buf;
	}

	protected double getFrequency() {
		return frequency;
	}

	//
	// CONSTRUCTOR
	//
	public CDT2WAVOut(byte[] buf, int freq) {
		super();
		this.buf = buf;
		this.frequency = freq;
        if (frequency == 44100.0)
            Z80_FREQ = 3400000.0;
        if (frequency == 22050.0)
            Z80_FREQ = 3494400.0;
		this.cycle = frequency / Z80_FREQ;
		init();
	}
	
	//
	// Concrete implementation interface
	//
	protected abstract void init();
	protected abstract void write(int numsamples);
	protected abstract void stop();

	/**
	 * Convert a sampling value in Z80 T-States to number of samples for wave output 
	 * @param tstates - number of Z80 T-States
	 * @return Number of wave samples at current frequency
	 */
	public int samples(int tstates) {
		return ( (int)(0.5 + (cycle*(double)tstates)) );
	}

	/**
	 * Toggles the sign of the wave
	 * TODO: WHOLE CONCEPT TO BE RECODED IN ToggleSgn();
	 */
	public void toggleAmp() {
		if (isLowAmp())
			amp = HIAMP;
        else
			amp = LOAMP;
	}
	
	/**
	 * Sets the sign of the wave
	 * @param high - true to set the wave to high amplitude
	 */
	public void setAmp(boolean high) {
		amp = (high ? HIAMP : LOAMP);
	}
	
	/**
	 * Is the wave sign currently low amplitude?
	 * @return boolean - true iff the wave sign currently low amplitude
	 */
	protected boolean isLowAmp() {
		return (amp == LOAMP);
	}

	/**
	 * Generate wave data for "len" samples.
	 * @param numsamples
	 */
	public void play(int numsamples) {
		write(numsamples);
	}

	/**
	 * Waits for a number of milliseconds
	 * @param ms - number of milliseconds
	 */
	public void pause(int ms) {
		int p;
		//if (curr!=(numblocks-1))
		//{
			p = (int)((((double) ms) * frequency)/1000.0);
			play(p);
		//}
	}
}
